@@ -11,7 +11,7 @@ class Map(object):
         function: str = None,
         **kwargs,
     ):
-        self._pipeline_module_name = function
+        self.module_id = function
         self.function = self._load_function(function)
 
     def run(self, instance: Instance, **kwargs):
