@@ -8,9 +8,13 @@ from alira.modules import module
 
 class RedisModule(module.Module):
     def __init__(
-        self, pipeline_module_name, model_identifier, configuration_directory, redis_server: str
+        self,
+        module_id,
+        model_identifier,
+        configuration_directory,
+        redis_server: str,
     ):
-        super().__init__(pipeline_module_name)
+        super().__init__(module_id)
 
         self.model_identifier = model_identifier
         self.configuration_directory = configuration_directory
@@ -23,3 +27,14 @@ class RedisModule(module.Module):
             return Queue(self.queue_name, connection=redis_connection, is_async=True)
 
         return None
+
+
+class ServiceException(Exception):
+    """
+    We raise this exception when we can't connect to a third-party, online service.
+    We expect this to happen whenever there's no internet connection.
+
+    This exception is useful to reattempt operations when they are being processed
+    by the Redis queue.
+    """
+    pass
\ No newline at end of file
