@@ -1,13 +1,13 @@
+from genericpath import isfile
 import boto3
 import uuid
 import logging
 import os
 
-from importlib import import_module
 from pathlib import Path
 
 from alira.instance import Instance
-from alira.modules.redis import RedisModule
+from alira.modules.redis import RedisModule, ServiceException
 
 from botocore.exceptions import ClientError, EndpointConnectionError
 
@@ -26,6 +26,14 @@ def aws_s3_upload(
     logging.info(
         f'Uploading {filename} to bucket "{aws_s3_bucket}" and location {s3_key}...'
     )
+
+    try:
+        with open(filename, "rb") as file:
+            buffer = file.read()
+    except FileNotFoundError as e:
+        logging.exception(f"There was an error loading the file {filename}")
+        return None
+
     try:
         session = boto3.Session(
             aws_access_key_id=aws_access_key,
@@ -34,17 +42,13 @@ def aws_s3_upload(
         )
         client = session.client("s3")
 
-        with open(filename, "rb") as file:
-            buffer = file.read()
-
         arguments = {"Bucket": aws_s3_bucket, "Key": s3_key, "Body": buffer}
-        content_type = "binary/octet-stream"
 
         extension = Path(filename).suffix.lower()
-
-        if extension in ["jpg", "jpeg"]:
+        content_type = "binary/octet-stream"
+        if extension in [".jpg", ".jpeg"]:
             content_type = "image/jpeg"
-        elif extension == "png":
+        elif extension == ".png":
             content_type = "image/png"
 
         arguments["ContentType"] = content_type
@@ -61,11 +65,7 @@ def aws_s3_upload(
 
     except EndpointConnectionError as e:
         logging.exception(e)
-
-        # If we can't connect to the service to upload the image, let's
-        # raise a RuntimeError so that the process can be retried as part
-        # of the Redis queue.
-        raise RuntimeError(e)
+        raise ServiceException(e) from e
     except ClientError as e:
         logging.exception(e)
     except Exception as e:
@@ -85,7 +85,7 @@ class S3(RedisModule):
         aws_access_key: str,
         aws_secret_key: str,
         aws_region_name: str,
-        id: str = None,
+        module_id: str = None,
         autogenerate_name: bool = False,
         redis_server: str = None,
         filtering: str = None,
@@ -94,7 +94,7 @@ class S3(RedisModule):
         **kwargs,
     ):
         super().__init__(
-            pipeline_module_name=PIPELINE_MODULE_NAME,
+            module_id=module_id or PIPELINE_MODULE_NAME,
             model_identifier=model_identifier,
             configuration_directory=configuration_directory,
             redis_server=redis_server,
@@ -115,7 +115,6 @@ class S3(RedisModule):
 
         self.aws_s3_public = aws_s3_public
 
-        self.id = id
         self.file = file
         self.aws_s3_upload_function = aws_s3_upload_function
 
@@ -124,33 +123,31 @@ class S3(RedisModule):
             logging.info(
                 f"The instance didn't pass the filtering criteria. Instance: {instance}"
             )
-            return None
-
-        if instance.image is None:
-            logging.info("The instance doesn't have an image")
-            return None
+            return {"status": "SKIPPED"}
 
         if self.file:
             filename = instance.get_attribute(self.file, default=instance.image)
         else:
-            filename = instance.image
-
-        filename = os.path.join(
-            self.configuration_directory,
-            self.model_identifier,
-            "images",
-            filename,
-        )
+            filename = os.path.join("images", instance.image)
 
         if self.autogenerate_name:
             _, file_extension = os.path.splitext(filename)
-            s3_key = f"{uuid.uuid4().hex}.{file_extension}"
+            s3_key = f"{uuid.uuid4().hex}{file_extension}"
         else:
-            s3_key = filename
+            s3_key = os.path.basename(filename)
 
         if self.aws_s3_key_prefix:
             s3_key = f"{self.aws_s3_key_prefix}/{s3_key}"
 
+        filename = os.path.join(self.configuration_directory, filename)
+
+        if not os.path.isfile(filename):
+            logging.info(f"The file {filename} does not exist")
+            return {
+                "status": "FAILURE",
+                "message": f"The file {filename} does not exist",
+            }
+
         arguments = {
             "aws_s3_bucket": self.aws_s3_bucket,
             "aws_s3_public": self.aws_s3_public,
@@ -168,7 +165,10 @@ class S3(RedisModule):
             else:
                 self.aws_s3_upload_function(**arguments)
 
-            result = {"s3_file_url": f"s3://{self.aws_s3_bucket}/{s3_key}"}
+            result = {
+                "status": "SUCCESS",
+                "s3_file_url": f"s3://{self.aws_s3_bucket}/{s3_key}",
+            }
 
             if self.aws_s3_public:
                 result[
@@ -178,20 +178,8 @@ class S3(RedisModule):
             return result
 
         except Exception as e:
-            print(e)
-            logging.exception("There was an error sending the notification email")
-            return None
-
-    def _load_function(self, function_name: str):
-        if function_name is None:
-            return None
-
-        try:
-            module_path, _, fn_name = function_name.rpartition(".")
-            function = getattr(import_module(module_path), fn_name)
-            logging.info(f"Loaded function {function_name}")
-
-            return function
-        except Exception as e:
-            logging.exception(f"Unable to load function {function_name}", e)
-            return None
\ No newline at end of file
+            logging.exception("There was an error uploading the image")
+            return {
+                "status": "FAILURE",
+                "message": f"Error uploading the image",
+            }
