@@ -92,8 +92,8 @@ class Pipeline(object):
         )
 
         for module in self.modules:
-            if hasattr(module, Pipeline.METADATA_PIPELINE_MODULE_NAME):
-                module_name = getattr(module, Pipeline.METADATA_PIPELINE_MODULE_NAME)
+            if hasattr(module, "module_id"):
+                module_name = module.module_id
             else:
                 module_name = f"{module.__module__}.{module.__class__.__name__}"
 
@@ -107,11 +107,7 @@ class Pipeline(object):
 
             output = module.run(instance=instance)
             if output is not None:
-                output_key = (
-                    f"{module_name}.{module.id}"
-                    if hasattr(module, "id") and module.id is not None
-                    else module_name
-                )
+                output_key = module_name
                 instance.properties[output_key] = output
 
             if output is not None and isinstance(output, dict):
